@@ -1,22 +1,11 @@
 import re
 import logging
-from datetime import datetime
-import sys
-
-import pytz
+from datetime import datetime, timezone
 
 from pyhamtools.consts import LookupConventions as const
 
 from pyhamtools.callsign_exceptions import callsign_exceptions
 
-UTC = pytz.UTC
-
-if sys.version_info < (2, 7, ):
-    class NullHandler(logging.Handler):
-        def emit(self, record):
-            pass
-
-
 class Callinfo(object):
     """
     The purpose of this class is to return data (country, latitude, longitude, CQ Zone...etc) for an
@@ -37,10 +26,7 @@ class Callinfo(object):
             self._logger = logger
         else:
             self._logger = logging.getLogger(__name__)
-            if sys.version_info[:2] == (2, 6):
-                self._logger.addHandler(NullHandler())
-            else:
-                self._logger.addHandler(logging.NullHandler())
+            self._logger.addHandler(logging.NullHandler())
 
         self._lookuplib = lookuplib
         self._callsign_info = None
@@ -70,7 +56,7 @@ class Callinfo(object):
         """
 
         callsign = callsign.upper()
-        homecall = re.search('[\d]{0,1}[A-Z]{1,2}\d([A-Z]{1,4}|\d{3,3}|\d{1,3}[A-Z])[A-Z]{0,5}', callsign)
+        homecall = re.search('[\\d]{0,1}[A-Z]{1,2}\\d([A-Z]{1,4}|\\d{3,3}|\\d{1,3}[A-Z])[A-Z]{0,5}', callsign)
         if homecall:
             homecall = homecall.group(0)
             return homecall
@@ -81,10 +67,10 @@ class Callinfo(object):
         """truncate call until it corresponds to a Prefix in the database"""
         prefix = callsign
         if timestamp is None:
-            timestamp = datetime.utcnow().replace(tzinfo=UTC)
+            timestamp = datetime.now(timezone.utc)
 
         if re.search('(VK|AX|VI)9[A-Z]{3}', callsign): #special rule for VK9 calls
-            if timestamp > datetime(2006,1,1, tzinfo=UTC):
+            if timestamp > datetime(2006,1,1, tzinfo=timezone.utc):
                 prefix = callsign[0:3]+callsign[4:5]
 
         while len(prefix) > 0:
@@ -115,7 +101,7 @@ class Callinfo(object):
 
         Args:
             callsign (str): Amateur Radio callsign
-            timestamp (datetime, optional): datetime in UTC (tzinfo=pytz.UTC)
+            timestamp (datetime, optional): datetime in UTC (tzinfo=timezone.utc)
 
         Raises:
             KeyError: Callsign could not be identified
@@ -124,12 +110,12 @@ class Callinfo(object):
         """
         entire_callsign = callsign.upper()
         if timestamp is None:
-            timestamp = datetime.utcnow().replace(tzinfo=UTC)
+            timestamp = datetime.now(timezone.utc)
 
-        if re.search('[/A-Z0-9\-]{3,15}', entire_callsign):  # make sure the call has at least 3 characters
+        if re.search('[/A-Z0-9\\-]{3,15}', entire_callsign):  # make sure the call has at least 3 characters
 
-            if re.search('\-\d{1,3}$', entire_callsign):  # cut off any -10 / -02 appendixes
-                callsign = re.sub('\-\d{1,3}$', '', entire_callsign)
+            if re.search('\\-\\d{1,3}$', entire_callsign):  # cut off any -10 / -02 appendixes
+                callsign = re.sub('\\-\\d{1,3}$', '', entire_callsign)
 
             if re.search('/[A-Z0-9]{1,4}/[A-Z0-9]{1,4}$', callsign):
                 callsign = re.sub('/[A-Z0-9]{1,4}$', '', callsign)  # cut off 2. appendix DH1TW/HC2/P -> DH1TW/HC2
@@ -192,20 +178,21 @@ class Callinfo(object):
                     data[const.BEACON] = True
                     return data
 
-                elif re.search('\d$', appendix):
-                    area_nr = re.search('\d$', appendix).group(0)
-                    callsign = re.sub('/\d$', '', callsign) #remove /number
-                    if len(re.findall(r'\d+', callsign)) == 1: #call has just on digit e.g. DH1TW
-                        callsign = re.sub('[\d]+', area_nr, callsign)
+                elif re.search('\\d$', appendix):
+                    area_nr = re.search('\\d$', appendix).group(0)
+                    callsign = re.sub('/\\d$', '', callsign) #remove /number
+                    if len(re.findall(r'\\d+', callsign)) == 1: #call has just on digit e.g. DH1TW
+                        callsign = re.sub('[\\d]+', area_nr, callsign)
                     else: # call has several digits e.g. 7N4AAL
-                        pass # no (two) digit prefix contries known where appendix would change entitiy
+                        pass # no (two) digit prefix countries known where appendix would change entity
                     return self._iterate_prefix(callsign, timestamp)
 
                 else:
                     return self._iterate_prefix(callsign, timestamp)
 
             # regular callsigns, without prefix or appendix
-            elif re.match('^[\d]{0,1}[A-Z]{1,2}\d([A-Z]{1,4}|\d{3,3}|\d{1,3}[A-Z])[A-Z]{0,5}$', callsign):
+            # elif re.match('^[\\d]{0,1}[A-Z]{1,2}\\d{1,2}[A-Z]{1,2}([A-Z]{1,4}|\\d{1,3})[A-Z]{0,5}$', callsign):
+            elif re.match('^[\\d]{0,1}[A-Z]{1,2}\\d{1,4}([A-Z]{1,4}|[A-Z]{1,2}\\d{0,3})[A-Z]{0,5}$', callsign):
                 return self._iterate_prefix(callsign, timestamp)
 
             # callsigns with prefixes (xxx/callsign)
@@ -214,8 +201,11 @@ class Callinfo(object):
                 pfx = re.sub('/', '', pfx.group(0))
                 #make sure that the remaining part is actually a callsign (avoid: OZ/JO81)
                 rest = re.search('/[A-Z0-9]+', entire_callsign)
-                rest = re.sub('/', '', rest.group(0))
-                if re.match('^[\d]{0,1}[A-Z]{1,2}\d([A-Z]{1,4}|\d{3,3}|\d{1,3}[A-Z])[A-Z]{0,5}$', rest):
+                if rest is None:
+                    self._logger.warning(u"non latin characters in callsign '{0}'".format(entire_callsign))
+                    raise KeyError
+                rest = re.sub('/', '', rest.group(0))  
+                if re.match('^[\\d]{0,1}[A-Z]{1,2}\\d([A-Z]{1,4}|\\d{3,3}|\\d{1,3}[A-Z])[A-Z]{0,5}$', rest):
                     return self._iterate_prefix(pfx)
 
         if entire_callsign in callsign_exceptions:
@@ -226,7 +216,7 @@ class Callinfo(object):
 
     def _lookup_callsign(self, callsign, timestamp=None):
         if timestamp is None:
-            timestamp = datetime.utcnow().replace(tzinfo=UTC)
+            timestamp = datetime.now(timezone.utc)
 
         # Check if operation is invalid
         invalid = False
@@ -274,7 +264,7 @@ class Callinfo(object):
 
         Args:
             callsign (str): Amateur Radio callsign
-            timestamp (datetime, optional): datetime in UTC (tzinfo=pytz.UTC)
+            timestamp (datetime, optional): datetime in UTC (tzinfo=timezone.utc)
 
         Returns:
             dict: Dictionary containing the callsign specific data
@@ -311,7 +301,7 @@ class Callinfo(object):
         callsign = callsign.upper()
 
         if timestamp is None:
-            timestamp = datetime.utcnow().replace(tzinfo=UTC)
+            timestamp = datetime.now(timezone.utc)
 
         callsign_data = self._lookup_callsign(callsign, timestamp)
 
@@ -328,7 +318,7 @@ class Callinfo(object):
 
         Args:
             callsign (str): Amateur Radio callsign
-            timestamp (datetime, optional): datetime in UTC (tzinfo=pytz.UTC)
+            timestamp (datetime, optional): datetime in UTC (tzinfo=timezone.utc)
 
         Returns:
             bool: True / False
@@ -344,7 +334,7 @@ class Callinfo(object):
 
         """
         if timestamp is None:
-            timestamp = datetime.utcnow().replace(tzinfo=UTC)
+            timestamp = datetime.now(timezone.utc)
 
         try:
             if self.get_all(callsign, timestamp):
@@ -357,7 +347,7 @@ class Callinfo(object):
 
         Args:
             callsign (str): Amateur Radio callsign
-            timestamp (datetime, optional): datetime in UTC (tzinfo=pytz.UTC)
+            timestamp (datetime, optional): datetime in UTC (tzinfo=timezone.utc)
 
         Returns:
             dict: Containing Latitude and Longitude
@@ -384,7 +374,7 @@ class Callinfo(object):
 
         """
         if timestamp is None:
-            timestamp = datetime.utcnow().replace(tzinfo=UTC)
+            timestamp = datetime.now(timezone.utc)
 
         callsign_data = self.get_all(callsign, timestamp=timestamp)
         return {
@@ -397,7 +387,7 @@ class Callinfo(object):
 
         Args:
             callsign (str): Amateur Radio callsign
-            timestamp (datetime, optional): datetime in UTC (tzinfo=pytz.UTC)
+            timestamp (datetime, optional): datetime in UTC (tzinfo=timezone.utc)
 
         Returns:
             int: containing the callsign's CQ Zone
@@ -407,7 +397,7 @@ class Callinfo(object):
 
         """
         if timestamp is None:
-            timestamp = datetime.utcnow().replace(tzinfo=UTC)
+            timestamp = datetime.now(timezone.utc)
 
         return self.get_all(callsign, timestamp)[const.CQZ]
 
@@ -416,7 +406,7 @@ class Callinfo(object):
 
         Args:
             callsign (str): Amateur Radio callsign
-            timestamp (datetime, optional): datetime in UTC (tzinfo=pytz.UTC)
+            timestamp (datetime, optional): datetime in UTC (tzinfo=timezone.utc)
 
         Returns:
             int: containing the callsign's CQ Zone
@@ -429,7 +419,7 @@ class Callinfo(object):
 
         """
         if timestamp is None:
-            timestamp = datetime.utcnow().replace(tzinfo=UTC)
+            timestamp = datetime.now(timezone.utc)
 
         return self.get_all(callsign, timestamp)[const.ITUZ]
 
@@ -438,7 +428,7 @@ class Callinfo(object):
 
         Args:
             callsign (str): Amateur Radio callsign
-            timestamp (datetime, optional): datetime in UTC (tzinfo=pytz.UTC)
+            timestamp (datetime, optional): datetime in UTC (tzinfo=timezone.utc)
 
         Returns:
             str: name of the Country
@@ -456,7 +446,7 @@ class Callinfo(object):
 
         """
         if timestamp is None:
-            timestamp = datetime.utcnow().replace(tzinfo=UTC)
+            timestamp = datetime.now(timezone.utc)
 
         return self.get_all(callsign, timestamp)[const.COUNTRY]
 
@@ -465,7 +455,7 @@ class Callinfo(object):
 
         Args:
             callsign (str): Amateur Radio callsign
-            timestamp (datetime, optional): datetime in UTC (tzinfo=pytz.UTC)
+            timestamp (datetime, optional): datetime in UTC (tzinfo=timezone.utc)
 
         Returns:
             int: containing the country ADIF id
@@ -475,7 +465,7 @@ class Callinfo(object):
 
         """
         if timestamp is None:
-            timestamp = datetime.utcnow().replace(tzinfo=UTC)
+            timestamp = datetime.now(timezone.utc)
 
         return self.get_all(callsign, timestamp)[const.ADIF]
 
@@ -484,7 +474,7 @@ class Callinfo(object):
 
         Args:
             callsign (str): Amateur Radio callsign
-            timestamp (datetime, optional): datetime in UTC (tzinfo=pytz.UTC)
+            timestamp (datetime, optional): datetime in UTC (tzinfo=timezone.utc)
 
         Returns:
             str: continent identified
@@ -504,6 +494,6 @@ class Callinfo(object):
             - AN: Antarctica
         """
         if timestamp is None:
-            timestamp = datetime.utcnow().replace(tzinfo=UTC)
+            timestamp = datetime.now(timezone.utc)
 
         return self.get_all(callsign, timestamp)[const.CONTINENT]
