@@ -3,12 +3,13 @@ from datetime import datetime, timezone
 
 import ephem
 
-def latlong_to_locator (latitude, longitude):
+def latlong_to_locator (latitude, longitude, precision=6):
     """converts WGS84 coordinates into the corresponding Maidenhead Locator
 
         Args:
             latitude (float): Latitude
             longitude (float): Longitude
+            precision (int): 4,6,8 chars (default 6)
 
         Returns:
             string: Maidenhead locator
@@ -32,35 +33,54 @@ def latlong_to_locator (latitude, longitude):
 
     """
 
+    if precision < 4 or precision ==5 or precision == 7 or precision > 8:
+        return ValueError
+
     if longitude >= 180 or longitude <= -180:
         raise ValueError
 
     if latitude >= 90 or latitude <= -90:
         raise ValueError
 
-    longitude += 180;
-    latitude +=90;
+    longitude +=180
+    latitude +=90
 
-    locator = chr(ord('A') + int(longitude / 20))
-    locator += chr(ord('A') + int(latitude / 10))
-    locator += chr(ord('0') + int((longitude % 20) / 2))
-    locator += chr(ord('0') + int(latitude % 10))
-    locator += chr(ord('A') + int((longitude - int(longitude / 2) * 2) / (2 / 24)))
-    locator += chr(ord('A') + int((latitude - int(latitude / 1) * 1 ) / (1 / 24)))
+    # copied & adapted from github.com/space-physics/maidenhead
+    A = ord('A')
+    a = divmod(longitude, 20)
+    b = divmod(latitude, 10)
+    locator = chr(A + int(a[0])) + chr(A + int(b[0]))
+    lon = a[1] / 2.0
+    lat = b[1]
+    i = 1
+
+    while i < precision/2:
+        i += 1
+        a = divmod(lon, 1)
+        b = divmod(lat, 1)
+        if not (i % 2):
+            locator += str(int(a[0])) + str(int(b[0]))
+            lon = 24 * a[1]
+            lat = 24 * b[1]
+        else:
+            locator += chr(A + int(a[0])) + chr(A + int(b[0]))
+            lon = 10 * a[1]
+            lat = 10 * b[1]
 
     return locator
 
-def locator_to_latlong (locator):
+def locator_to_latlong (locator, center=True):
     """converts Maidenhead locator in the corresponding WGS84 coordinates
 
         Args:
-            locator (string): Locator, either 4 or 6 characters
+            locator (string): Locator, either 4, 6 or 8 characters
+            center (bool): Center of (sub)square. By default True. If False, the south/western corner will be returned
 
         Returns:
             tuple (float, float): Latitude, Longitude
 
         Raises:
-            ValueError: When called with wrong or invalid input arg
+            ValueError: When called with wrong or invalid Maidenhead locator string
             TypeError: When arg is not a string
 
         Example:
@@ -79,7 +99,7 @@ def locator_to_latlong (locator):
 
     locator = locator.upper()
 
-    if len(locator) == 5 or len(locator) < 4:
+    if len(locator) < 4 or len(locator) == 5 or len(locator) == 7:
         raise ValueError
 
     if ord(locator[0]) > ord('R') or ord(locator[0]) < ord('A'):
@@ -100,23 +120,44 @@ def locator_to_latlong (locator):
         if ord (locator[5]) > ord('X') or ord(locator[5]) < ord('A'):
             raise ValueError
 
+    if len(locator) == 8:
+        if ord(locator[6]) > ord('9') or ord(locator[6]) < ord('0'):
+            raise ValueError
+        if ord (locator[7]) > ord('9') or ord(locator[7]) < ord('0'):
+            raise ValueError
+
     longitude = (ord(locator[0]) - ord('A')) * 20 - 180
     latitude = (ord(locator[1]) - ord('A')) * 10 - 90
     longitude += (ord(locator[2]) - ord('0')) * 2
-    latitude += (ord(locator[3]) - ord('0'))
+    latitude += (ord(locator[3]) - ord('0')) * 1
 
-    if len(locator) == 6:
-        longitude += ((ord(locator[4])) - ord('A')) * (2 / 24)
-        latitude += ((ord(locator[5])) - ord('A')) * (1 / 24)
+    if len(locator) == 4:
 
-        # move to center of subsquare
-        longitude += 1 / 24
-        latitude += 0.5 / 24
+        if center:
+            longitude += 2 / 2
+            latitude += 1.0 / 2
+
+    elif len(locator) == 6:
+        longitude += (ord(locator[4]) - ord('A')) * 5.0 / 60
+        latitude += (ord(locator[5]) - ord('A')) * 2.5 / 60
+
+        if center:
+            longitude += 5.0 / 60 / 2
+            latitude += 2.5 / 60 / 2
+
+    elif len(locator) == 8:
+        longitude += (ord(locator[4]) - ord('A')) * 5.0 / 60
+        latitude += (ord(locator[5]) - ord('A')) * 2.5 / 60
+
+        longitude += int(locator[6]) * 5.0 / 600
+        latitude += int(locator[7]) * 2.5 / 600
+
+        if center:
+            longitude += 5.0 / 600 / 2
+            latitude += 2.5 / 600 / 2
 
     else:
-        # move to center of square
-        longitude += 1;
-        latitude += 0.5;
+        raise ValueError
 
     return latitude, longitude
 
@@ -125,14 +166,14 @@ def calculate_distance(locator1, locator2):
     """calculates the (shortpath) distance between two Maidenhead locators
 
         Args:
-            locator1 (string): Locator, either 4 or 6 characters
-            locator2 (string): Locator, either 4 or 6 characters
+            locator1 (string): Locator, either 4, 6 or 8 characters
+            locator2 (string): Locator, either 4, 6 or 8 characters
 
         Returns:
             float: Distance in km
 
         Raises:
-            ValueError: When called with wrong or invalid input arg
+            ValueError: When called with wrong or invalid maidenhead locator strings
             AttributeError: When args are not a string
 
         Example:
@@ -142,6 +183,9 @@ def calculate_distance(locator1, locator2):
            >>> calculate_distance("JN48QM", "QF67bf")
            16466.413
 
+        Note:
+            Distances is calculated between the centers of the (sub) squares
+
     """
 
     R = 6371 #earh radius
@@ -160,15 +204,15 @@ def calculate_distance(locator1, locator2):
     c = 2 * atan2(sqrt(a), sqrt(1-a))
     d = R * c #distance in km
 
-    return d;
+    return d
 
 
 def calculate_distance_longpath(locator1, locator2):
     """calculates the (longpath) distance between two Maidenhead locators
 
         Args:
-            locator1 (string): Locator, either 4 or 6 characters
-            locator2 (string): Locator, either 4 or 6 characters
+            locator1 (string): Locator, either 4, 6 or 8 characters
+            locator2 (string): Locator, either 4, 6 or 8 characters
 
         Returns:
             float: Distance in km
@@ -184,6 +228,8 @@ def calculate_distance_longpath(locator1, locator2):
            >>> calculate_distance_longpath("JN48QM", "QF67bf")
            23541.5867
 
+        Note:
+            Distance is calculated between the centers of the (sub) squares
     """
 
     c = 40008 #[km] earth circumference
@@ -196,8 +242,8 @@ def calculate_heading(locator1, locator2):
     """calculates the heading from the first to the second locator
 
         Args:
-            locator1 (string): Locator, either 4 or 6 characters
-            locator2 (string): Locator, either 4 or 6 characters
+            locator1 (string): Locator, either 4, 6 or 8 characters
+            locator2 (string): Locator, either 4, 6 or 6 characters
 
         Returns:
             float: Heading in deg
@@ -213,6 +259,9 @@ def calculate_heading(locator1, locator2):
            >>> calculate_heading("JN48QM", "QF67bf")
            74.3136
 
+        Note:
+            Heading is calculated between the centers of the (sub) squares
+
     """
 
     lat1, long1 = locator_to_latlong(locator1)
@@ -236,8 +285,8 @@ def calculate_heading_longpath(locator1, locator2):
     """calculates the heading from the first to the second locator (long path)
 
         Args:
-            locator1 (string): Locator, either 4 or 6 characters
-            locator2 (string): Locator, either 4 or 6 characters
+            locator1 (string): Locator, either 4, 6 or 8 characters
+            locator2 (string): Locator, either 4, 6 or 8 characters
 
         Returns:
             float: Long path heading in deg
@@ -253,6 +302,9 @@ def calculate_heading_longpath(locator1, locator2):
            >>> calculate_heading_longpath("JN48QM", "QF67bf")
            254.3136
 
+        Note:
+            Distance is calculated between the centers of the (sub) squares
+
     """
 
     heading = calculate_heading(locator1, locator2)
@@ -265,7 +317,7 @@ def calculate_sunrise_sunset(locator, calc_date=None):
     """calculates the next sunset and sunrise for a Maidenhead locator at a give date & time
 
         Args:
-            locator1 (string): Maidenhead Locator, either 4 or 6 characters
+            locator1 (string): Maidenhead Locator, either 4, 6 or 8 characters
             calc_date (datetime, optional): Starting datetime for the calculations (UTC)
 
         Returns:
